%% Blind Source Separation, Skript 3
% In diesem Skript wird zustzlich Beamforming eingesetzt, um die Signale vor 
% der eigentlichen ICA zustzlich zu filtern.
%% Initialisierung
%%
clear all
close all
clc
run setupPath.m

%% Initialisierung
samplerate = 8000;       % Abtastungen pro Sekunde in Hz
bfDirections = [10; 30];    % Spezifizuerung der Richtungen der...
                            %    Signalquellen zwecks Beamforming
receiverDistance  = 0.081;     % m

%% STFT-Konfiguration
windowType   = 'hamming';    % aus {'hamming', 'hann', 'gauss', 'tukey'}
windowLength = 1024;         % Abtastungen
windowHop    =  256;         % Abtastungen
fftLength    = 1024;         % Abtastungen
%% Vorbereitung der Aufnahmen
%%
%% Lesen Sie die Aufnahmen ein mittels des Befehls audioread


%% Normalisieren Sie die Amplutiden

% Berechnen Sie einen Zeitvektor basierend auf der ...
%   Signallnge und der Abtastrate zum Plotten

%% Wenden Sie Beamforming an, um zwei Mischsignale fr ICA zu erhalten (vorgegeben)
%%
beamformer = Beamformer([0 0 0], receiverDistance, samplerate);
x1 = beamformer.getBeamformedSignal([u1;u2;u3;u4], bfDirections(1));
x2 = beamformer.getBeamformedSignal([u1;u2;u3;u4], bfDirections(2));
timevector_x = 0 : 1/samplerate : (length(x1)-1)/samplerate; % a time vector for plotting beamformed signals

%% Wenden Sie STFT auf die Mischsignale an
%%
stft = STFT(windowLength, windowHop, windowType, fftLength, samplerate);
[X1, timeslots, freqbins] = stft.stft(x1);
X2 = stft.stft(x2);
numFreqbins = length(freqbins);
numTimeslots = length(timeslots);
%% Wenden Sie die ICA an und erhalten Sie A und W
%%
Y1 = zeros(size(X1));
Y2 = zeros(size(X2));
W = zeros(2, 2, numFreqbins);
A = zeros(2, 2, numFreqbins);
for count_freqbin = 1 : numFreqbins
    [Ytemp, Q, T, ~] = ...
        fastICA([X1(count_freqbin,:); X2(count_freqbin,:)], 2, 'kurtosis', 0);
    A(:,:,count_freqbin) = inv(Q*T); % Erhalten Sie die Mischmatrix
    W(:,:,count_freqbin) = Q*T; % Erhalten Sie die Wiederherstellungsmatrix
    Y1(count_freqbin,:) = Ytemp(1,:); % Erhalten Sie das wiederhergestellte Signal
    Y2(count_freqbin,:) = Ytemp(2,:); % Erhalten Sie das wiederhergestellte Signal
end
%% Permutationsausrichtung und Skalierungskorrektur
%%
perm = permutationAlignmentDyadic(A, Y1, Y2, samplerate, 1);

for count_freqbin = 1 : numFreqbins
    % fr jeden Frequenzbin fhren Sie PA und SK aus
    W(:,:,count_freqbin) = perm(:,:,count_freqbin)*W(:,:,count_freqbin);
    
    % Wenden Sie PA auf jeden Frequenz-Bin von W an
    
    % Wenden Sie SK auf jeden Frequenz-Bin von W an

    % Trennung
    Ytemp = W(:,:,count_freqbin)*centerRows([X1(count_freqbin,:); X2(count_freqbin,:)]);
    Y1(count_freqbin,:) = Ytemp(1,:);
    Y2(count_freqbin,:) = Ytemp(2,:);
end
clear freqbin;

%% Wenden Sie ISTFT an, um Signale in den Zeitbereich zu transformieren
%%
[y1, timevector_y] = stft.istft(Y1);
y2 = stft.istft(Y2);

%% Evaluieren Sie die Performance durch Plotten der Quellsignale, Mischsignale und wiederhergestellten Signale
%%

%% Hren Sie sich die Signale an mit der Funktion sound(signal, sample_rate_of_signal)
%%